@@ -98,14 +98,13 @@ Main difference with using `print!(format!( ... ))`:
 
 You'll find more examples and advice in the *templates* example.
 
-Note that there's no macro yet supporting templates for whole markdown *texts* but they should be available soon.
-
 # Examples
 
 The repository contains several other examples, which hopefully cover the whole API while being simple enough. It's recommended you start by trying them or at least glance at their code.
 
 */
 
+mod ask;
 mod area;
 mod code;
 mod color;
@@ -126,9 +125,11 @@ mod str_fit;
 mod styled_char;
 mod tbl;
 mod text;
+mod tokens;
 mod views;
 mod wrap;
 
+pub use ask::*;
 pub use area::{compute_scrollbar, terminal_size, Area};
 pub use color::{ansi, gray, rgb};
 pub use composite::FmtComposite;
@@ -146,6 +147,7 @@ pub use str_fit::StrFit;
 pub use spacing::Spacing;
 pub use styled_char::StyledChar;
 pub use text::FmtText;
+use tokens::*;
 pub use views::{
     InputField, ListView, ListViewCell, ListViewColumn, MadView, ProgressBar, TextView,
 };
