@@ -6,7 +6,7 @@ use crate::styled_char::StyledChar;
 ///
 /// For the default styling only the fg color is defined
 ///  and the char is ▐ but everything can be changed.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct ScrollBarStyle {
     pub track: StyledChar,
     pub thumb: StyledChar,
